"""Django settings file interaction for determining Vanilla Framework middleware settings."""
from django.conf import settings


def get_version():
    """
    Get the version of Vanilla Framework to use.

    If no version is provided in settings, use the latest available.
    """
    return getattr(settings, "VANILLAFRAMEWORK_VERSION", "3.8.1")


def get_min_css_url(ver=None):
    """
    Get the min.css CDN URL for the provided version of Vanilla Framework.

    If a custom URL or version is provided, use that instead.
    """
    return getattr(settings, "VANILLAFRAMEWORK_MIN_CSS_URL",
                   f"https://assets.ubuntu.com/v1/vanilla-framework-version-{ver if ver else get_version()}.min.css")


def get_local_css_path(ver=None):
    """Get the static file path of the local copy of Vanilla Framework."""
    return getattr(settings, "VANILLAFRAMEWORK_CSS_PATH",
                   f"css/vanilla-framework-version-{ver if ver else get_version()}.min.css")


def get_local_sass_path():
    """Get the static file path of the custom sass file that includes Vanilla Framework."""
    return getattr(settings, "VANILLAFRAMEWORK_PATH", "vanillaframework.scss")



"""Django template tag manager for Vanilla Framework."""

from django import template
from ..config import get_min_css_url, get_local_css_path, get_local_sass_path
from ..util import has_local_css

register = template.Library()


@register.inclusion_tag('vanillaframework/tags/css.html')
def vanillaframework_css():
    """Build the css tag for using Vanilla Framework with Django templates."""
    return {
        'is_local': has_local_css(),
        'vanilla_css_path': get_local_css_path() if has_local_css() else get_min_css_url()
    }


@register.inclusion_tag('vanillaframework/tags/sass.html')
def vanillaframework_sass():
    """Build the sass tag for using a customized Vanilla Framework with Django templates."""
    return {
        'vanilla_sass_path': get_local_sass_path()
    }



from django.template import Context, Template
from django.core.management import call_command
from .util import clean_test_files

css_template = Template(
    """
    {% load vanillaframework_tags %}
    {% vanillaframework_css %}
    """
)

sass_template = Template(
    """
    {% load vanillaframework_tags %}
    {% vanillaframework_sass %}
    """
)


def test_default_cdn_css_with_version(settings):
    clean_test_files()
    settings.VANILLAFRAMEWORK_VERSION = "3.8.1"
    output = css_template.render(Context({}))

    assert '<link rel="stylesheet" href="https://assets.ubuntu.com/v1/vanilla-framework-version-3.8.1.min.css" />' in output


def test_default_local_css_with_version(settings):
    test_version = "1.0.0"
    call_command("vanillaframework", "-i", "--css", "--use-version", test_version)
    settings.VANILLAFRAMEWORK_VERSION = test_version
    output = css_template.render(Context({}))

    assert f'<link rel="stylesheet" href="/static/css/vanilla-framework-version-{test_version}.min.css" />' in output
    clean_test_files()


def test_default_local_sass_with_version(settings):
    test_version = "1.0.0"
    call_command("vanillaframework", "-i", "--use-version", test_version)
    settings.VANILLAFRAMEWORK_VERSION = test_version
    settings.STATICFILES_FINDERS = (
        'django.contrib.staticfiles.finders.FileSystemFinder',
        'compressor.finders.CompressorFinder'
    )
    settings.COMPRESS_PRECOMPILERS = (
        ('text/x-scss', 'django_libsass.SassCompiler'),
    )
    output = sass_template.render(Context({}))

    assert '<link rel="stylesheet" href="/static/CACHE/css/' in output
    assert 'type="text/css"' in output
    clean_test_files()



"""
Django settings for django-vanillaframework tests.
"""

from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent
SECRET_KEY = 'insecure-secret-key'
DEBUG = False

ALLOWED_HOSTS = ["*"]


# Application definition

INSTALLED_APPS = [
    'django.contrib.staticfiles',
    'vanillaframework',
    'compressor'
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': []
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
            ],
        },
    },
]

DATABASES = {}

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_TZ = True

STATIC_URL = 'static/'
STATIC_ROOT = BASE_DIR / "static/"

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'



"""django-vanillaframework test utilities."""
import shutil
from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent


def clean_test_files():
    """Clean files generated by tests."""
    try:
        shutil.rmtree(BASE_DIR / "static")
    except FileNotFoundError:
        pass



"""Vanilla Framework management command."""
import os.path
import subprocess
from django.core.management.base import BaseCommand
from django.conf import settings
from pathlib import Path
import urllib.request
from vanillaframework import __version__, config


class Command(BaseCommand):
    """Vanilla Framework management command class."""

    help = "Manage Vanilla Framework install within the Django project."

    def add_arguments(self, parser):
        """Add -- args to the management command."""
        parser.add_argument("-i", "--install", action='store_true', help="Install Vanilla Framework locally")
        parser.add_argument("--css", action='store_true',
                            help="Use the min.css version of Vanilla Framework (no customization)")
        parser.add_argument("--use-version", type=str,
                            help="The version of Vanilla Framework to use, defaults to settings version or newest")
        parser.add_argument("--static-folder", type=str, help="The static folder to put Vanilla Framework in",
                            default=settings.STATIC_ROOT)

    def handle(self, *args, **options):
        """Handle Vanilla Framework management commands."""
        if options["install"]:
            self.handle_install(*args, **options)

    def handle_install(self, *args, **options):
        """Handle Vanilla Framework install command."""
        version_to_get = options["use_version"] if "use_version" in options and options[
            "use_version"] is not None else config.get_version()

        if "static_folder" not in options:
            print("Error: no static folder provided. Use --static-folder or add a STATIC_ROOT to settings.")
            return

        if options["css"]:
            Path(os.path.join(options["static_folder"], 'css')).mkdir(parents=True, exist_ok=True)
            urllib.request.urlretrieve(config.get_min_css_url(version_to_get), os.path.join(options["static_folder"],
                                                                                            config.get_local_css_path(
                                                                                                version_to_get)))
        else:
            Path(options["static_folder"]).mkdir(parents=True, exist_ok=True)
            npm_result = subprocess.run(
                ["npm", "install", "--prefix", options['static_folder'], f"vanilla-framework@{version_to_get}"])

            if npm_result.returncode != 0:
                print("Error: npm failed to download vanilla-framework. Make sure you have npm installed and in your")
                print("PATH (https://docs.npmjs.com/downloading-and-installing-node-js-and-npm)")
                return

            vanilla_filename = os.path.join(options["static_folder"], config.get_local_sass_path())
            if Path(vanilla_filename).is_file():
                print(f"Note: {config.get_local_sass_path()} already exists, leaving it alone.")
                print()
            else:
                vanilla_file = open(vanilla_filename, "w")
                vanilla_file.write(
                    """// Override default Vanilla settings here,


                    // Import the framework
                    @import 'node_modules/vanilla-framework/scss/build.scss';

                    // Include Vanilla Framework
                    @include vanilla;
                    """.replace("                    ", ""))
                vanilla_file.close()

            if 'compressor' not in settings.INSTALLED_APPS:
                print("The sass version of vanilla-framework uses django-compressor. Add 'compressor' to your")
                print("INSTALLED_APPS list in your settings file, and make sure django-compressor is installed")
                print()

            scss_precomp_available = False
            if hasattr(settings, "COMPRESS_PRECOMPILERS"):
                for precompiler in settings.COMPRESS_PRECOMPILERS:
                    if precompiler[0] == 'text/x-scss':
                        scss_precomp_available = True
                        break

            if not scss_precomp_available:
                print("The sass version of vanilla-framework needs a sass precompiler to work with Django. Install")
                print("one, such as django-libsass, then add it to your COMPRESS_PRECOMPILERS list in settings.")
                print("For example, using django-libsass, add:")
                print("COMPRESS_PRECOMPILERS = (")
                print("    ('text/x-scss', 'django_libsass.SassCompiler'),")
                print(")")
                print()

        if "use_version" in options and options["use_version"] is not None:
            print("You have downloaded a specific version of Vanilla Framework. To use it, add:")
            print(f"VANILLAFRAMEWORK_VERSION = {version_to_get}")
            print("to your settings file.")

    def get_version(self):
        """Get the django-vanillaframework version."""
        return __version__



"""Utility functions for handling Vanilla Framework in Django."""
import os.path
from pathlib import Path
from django.conf import settings
from .config import get_local_css_path


def has_local_css():
    """Check if there is a local copy of Vanilla Framework in the correct location and return True if so."""
    css = os.path.join(settings.STATIC_ROOT, get_local_css_path()) if hasattr(settings,
                                                                              "STATIC_ROOT") else get_local_css_path()
    return Path(css).is_file()



"""Setup.py file."""
from setuptools import setup, find_packages
from pathlib import Path
from vanillaframework import __version__

setup(
    name='django-vanillaframework',
    version=__version__,
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    url='https://github.com/lvoytek/django-vanillaframework',
    license='LGPL v3',
    author='Lena Voytek',
    author_email='lena@voytek.dev',
    description='Vanilla Framework frontend for Django',
    long_description=Path(Path(__file__).parent / "README.md").read_text(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'django>=3.2'
    ]
)
